#include <windows.h>
#include <httpext.h>
#include <process.h>
#include "MyISAPIMsg\MyISAPIMsg.h"
#include <stdio.h>
#include <ATLBASE.H>
#include "atl7extr.h"



#define REG_PATH TEXT("Software\\Test\\MyISAPI")
#define REG_ISAPI_THREADS_POOL TEXT("ISAPIThreadsPool")
#define REG_ISAPI_QUEUE_TIMEOUT TEXT("ISAPIQueueTimeout")
#define REG_ISAPI_QUEUE_SIZE TEXT("ISAPIQueueSize")

#define DefaultTimeout 60000
#define DefaultSize 60
#define DefaultPool 5

#define VERSION_MINOR 0
#define VERSION_MAJOR 1
const char szDescription[]="Test ISAPI Extension";

#define EventSource TEXT("Test.MyISAPI")

//  http
static const char szRespOK[]="200 OK";

void ExtensionThreadProc(LPVOID);

HANDLE hSemaphore=NULL; // 
DWORD dwMaxThreads=DefaultPool; //  
DWORD dwQueueTimeout=DefaultTimeout; //   
DWORD dwQueueSize=DefaultSize; // 
HANDLE hEvt=NULL; // 
HANDLE hIOPort=NULL;

//CacheIsapiExtensions flag bugfix
CComAutoCriticalSection g_cs; //initialization synchronyzer
long g_lInitCount(0); //attached threads counter
BOOL g_bInitResult(FALSE);


BOOL WINAPI GetExtensionVersion(HSE_VERSION_INFO* pVer)
{
  // 
  pVer->dwExtensionVersion=MAKELONG(VERSION_MINOR, VERSION_MAJOR);
  // 
  memcpy(pVer->lpszExtensionDesc, szDescription, sizeof(szDescription));
  CComCritSecLock<CComAutoCriticalSection> lock(g_cs);
  if (g_lInitCount++ > 0)
	  return g_bInitResult;
  HKEY hkey;
  //  
  if((hEvt = ::RegisterEventSource(NULL, EventSource)) == NULL)
  {
    //  
    ::OutputDebugString(TEXT("MyISAPI: Failed to register event source.\n"));
    return FALSE;
  }
  //  
  if(::RegOpenKeyEx(HKEY_LOCAL_MACHINE, REG_PATH, 0, KEY_READ, &hkey) == ERROR_SUCCESS)
  {
    DWORD dwSize = 4;
    DWORD dwType;
    //   
    if(::RegQueryValueEx(hkey, REG_ISAPI_THREADS_POOL, NULL, &dwType, (unsigned char *)&dwMaxThreads, &dwSize)!=ERROR_SUCCESS || dwType!=REG_DWORD || !dwMaxThreads)
    {
      //   
      dwMaxThreads = DefaultPool;
      dwSize = 4;
    }
    //  
    if(::RegQueryValueEx(hkey, REG_ISAPI_QUEUE_TIMEOUT, NULL, &dwType, (unsigned char *)&dwQueueTimeout, &dwSize)!=ERROR_SUCCESS || dwType!=REG_DWORD || !dwQueueTimeout)
    {
      dwQueueTimeout = DefaultTimeout;
      dwSize = 4;
    }
    if(::RegQueryValueEx(hkey, REG_ISAPI_QUEUE_SIZE, NULL, &dwType, (unsigned char *)&dwQueueSize, &dwSize)!=ERROR_SUCCESS || dwType!=REG_DWORD || !dwQueueSize) dwQueueSize = DefaultSize;
    ::RegCloseKey(hkey);
  }
  // IO Completion port
  if(hIOPort = ::CreateIoCompletionPort(INVALID_HANDLE_VALUE, NULL, 0, 0))
  {
    // 
    if(hSemaphore = ::CreateSemaphore(NULL, dwQueueSize, dwQueueSize, NULL))
    {
      for(DWORD i=0; i < dwMaxThreads; i++)
        if(_beginthread(ExtensionThreadProc,0,NULL) == -1)
          ::ReportEvent(hEvt, EVENTLOG_ERROR_TYPE, 0, MSG_ISAPI_THREAD_FAILED, NULL, 0, 0, NULL, NULL); //   

      //    
      ::ReportEvent(hEvt, EVENTLOG_INFORMATION_TYPE, 0, MSG_ISAPI_INITIALIZED, NULL, 0, 0, NULL, NULL);
      g_bInitResult = TRUE;
      return TRUE;
    }
  }
  //   
  ::ReportEvent(hEvt, EVENTLOG_ERROR_TYPE, 0, MSG_ISAPI_INITIALIZATION_FAILED, NULL, 0, 0, NULL, NULL);
  return FALSE;
}

BOOL WINAPI TerminateExtension(DWORD dwFlags)
{
  CComCritSecLock<CComAutoCriticalSection> lock(g_cs);
  if (--g_lInitCount > 0)
	  return TRUE;
  if(hIOPort)
  	::CloseHandle(hIOPort);
  // 
  if(hSemaphore)
  	::CloseHandle(hSemaphore);
  //    
  ::ReportEvent(hEvt,EVENTLOG_INFORMATION_TYPE,0,MSG_ISAPI_UNINITIALIZED,NULL,0,0,NULL,NULL);
  //  
  ::DeregisterEventSource(hEvt);
  return TRUE;
}

DWORD WINAPI HttpExtensionProc(LPEXTENSION_CONTROL_BLOCK lpECB)
{
  //     
  if(::WaitForSingleObject(hSemaphore,dwQueueTimeout) == WAIT_TIMEOUT)
  {
    // 
    ::ReportEvent(hEvt, EVENTLOG_ERROR_TYPE, 0, MSG_ISAPI_QUEUE_TIMEOUT, NULL, 0, 0, NULL, NULL);
    return HSE_STATUS_ERROR;
  }
  //   Extension Control Block    overlapped
  if(!PostQueuedCompletionStatus(hIOPort,-1,NULL,(LPOVERLAPPED)lpECB))
  {
    ::ReleaseSemaphore(hSemaphore,1,NULL);
    return HSE_STATUS_ERROR;
  }
  //   
  return HSE_STATUS_PENDING;
}

void ExtensionThreadProc(LPVOID)
{
  DWORD dwFlag, dwNull;
  OVERLAPPED * pParam;
  while(::GetQueuedCompletionStatus(hIOPort, &dwFlag, &dwNull, &pParam, INFINITE) && dwFlag != NULL)
  {
    // 
    LPEXTENSION_CONTROL_BLOCK lpECB = (LPEXTENSION_CONTROL_BLOCK)pParam;
    // a  b
    __int64 ia, ib;
    //  
    int iParamFlg = 0;
    //    
    char *ppos = lpECB->lpszQueryString;
    char *pend = strchr(ppos, 0);
    ppos--;
    while(ppos && ppos + 3 < pend)
    {
      ppos++;
      char cPar = *ppos++;
      //   "X="
      if(*ppos++ == '=')
      {
        switch(cPar)
        {
          case 'a':
              ia = atoi(ppos);
              iParamFlg |= 0x01;
              break;
          case 'b':
              ib = atoi(ppos);
              iParamFlg |= 0x02;
        }
      }
      //  .
      ppos = strchr(ppos, '&');
    }
    //  
    char *pAll = NULL;
    // 
    DWORD dwAll;
    if(iParamFlg == 3) // 
    {
      pAll = new char[21];
      __int64 res = ia*ib;
      sprintf(pAll, "%I64i", res);
      dwAll = strlen(pAll);
    }
    else
    {
      dwAll = 0;
      //   
      lpECB->GetServerVariable(lpECB->ConnID, "ALL_RAW", pAll, (unsigned long *)&dwAll); //get size
      pAll = new char[dwAll];
      // 
      lpECB->GetServerVariable(lpECB->ConnID, "ALL_RAW", pAll, (unsigned long *)&dwAll); //get data
    }
    char pszHdr[200];
    static char szHeader[] = "Content-Length: %lu\r\nContent-type: text/plain\r\nPragma: no-cache\r\nExpires: 0\r\nCache-Control: no-cache\r\n\r\n";
    // 
    sprintf(pszHdr, szHeader, dwAll); //create header
    HSE_SEND_HEADER_EX_INFO hInfo;
    hInfo.pszStatus = szRespOK;
    hInfo.pszHeader = pszHdr;
    hInfo.cchStatus = sizeof(szRespOK) - 1;
    hInfo.cchHeader = strlen(pszHdr);
    // KeepAlive http/1.1
    hInfo.fKeepConn = TRUE;
    // 
    lpECB->ServerSupportFunction(lpECB->ConnID, HSE_REQ_SEND_RESPONSE_HEADER_EX, &hInfo, NULL, NULL);
    // 
    lpECB->WriteClient(lpECB->ConnID, pAll, &dwAll, HSE_IO_SYNC);
    DWORD dwStatus = HSE_STATUS_SUCCESS_AND_KEEP_CONN;
    //    
    lpECB->ServerSupportFunction(lpECB->ConnID, HSE_REQ_DONE_WITH_SESSION, &dwStatus, NULL, NULL);
    delete [] pAll;
    // 
    ::ReleaseSemaphore(hSemaphore, 1, NULL);
  }
  _endthread();
}
